@@ -85,7 +85,6 @@ static void          fo_doc_cairo_rect_filled         (FoDoc       *fo_doc,
 						       gdouble      height);
 static void          fo_doc_cairo_fill                (FoDoc       *fo_doc);
 static void          fo_doc_cairo_stroke              (FoDoc       *fo_doc);
-static void          fo_doc_cairo_fill_stroke         (FoDoc       *fo_doc);
 static void          fo_doc_cairo_place_image         (FoDoc       *fo_doc,
 						       FoImage     *fo_image,
 						       gdouble      x,
@@ -212,7 +211,6 @@ fo_doc_cairo_base_init (FoDocCairoClass *klass)
   fo_doc_class->rect_filled         = fo_doc_cairo_rect_filled;
   fo_doc_class->fill                = fo_doc_cairo_fill;
   fo_doc_class->stroke              = fo_doc_cairo_stroke;
-  fo_doc_class->fill_stroke         = fo_doc_cairo_fill_stroke;
   fo_doc_class->place_image         = fo_doc_cairo_place_image;
   fo_doc_class->render_layout_lines = fo_doc_cairo_render_layout_lines;
   fo_doc_class->render_layout       = fo_doc_cairo_render_layout;
@@ -274,6 +272,8 @@ fo_doc_cairo_new (void)
  * @error:         #GError that is set if an error occurs.
  * 
  * Open @filename as the output file for @fo_doc.
+ *
+ * No reference to @libfo_context is kept.
  **/
 void
 fo_doc_cairo_open_file (FoDoc          *fo_doc,
@@ -453,7 +453,7 @@ fo_doc_cairo_begin_page (FoDoc   *fo_doc,
 
       fo_doc_cairo->current_filename =
 	g_strdup_printf ("%*s.%02d%s",
-			 (g_strrstr (fo_doc_cairo->base_filename, ".") -
+			 (int) (g_strrstr (fo_doc_cairo->base_filename, ".") -
 			  fo_doc_cairo->base_filename),
 			 fo_doc_cairo->base_filename,
 			 fo_doc_cairo->output_sequence,
@@ -762,7 +762,7 @@ fo_doc_cairo_set_dash (FoDoc  *fo_doc,
 		       gdouble b,
 		       gdouble w)
 {
-  gdouble values[] = {b,w};
+  const gdouble values[] = {b,w};
   gint n = ((b == 0.0) && (w == 0.0))? 0 : 2;
 
   g_return_if_fail (FO_IS_DOC_CAIRO (fo_doc));
@@ -996,74 +996,126 @@ fo_doc_cairo_stroke (FoDoc *fo_doc)
 }
 
 /**
- * fo_doc_cairo_fill_stroke:
- * @fo_doc: #FoDoc.
+ * fo_doc_cairo_place_image:
+ * @fo_doc: 
+ * @fo_image: 
+ * @x: 
+ * @y: 
+ * @xscale: 
+ * @yscale: 
+ * 
  * 
- * Fill and stroke the path of @fo_doc with the current fill and
- * stroke colors.
  **/
-void
-fo_doc_cairo_fill_stroke (FoDoc *fo_doc)
-{
-  g_return_if_fail (FO_IS_DOC_CAIRO (fo_doc));
-  g_return_if_fail (FO_DOC_CAIRO (fo_doc)->cr != NULL);
-
-  /*PDF_fill_stroke (FO_DOC_CAIRO (fo_doc)->cr);*/
-}
-
 static void
 fo_doc_cairo_place_image (FoDoc   *fo_doc,
 			  FoImage *fo_image,
-			  gdouble  x,
-			  gdouble  y,
-			  gdouble  xscale,
-			  gdouble  yscale)
+			  gdouble  x G_GNUC_UNUSED,
+			  gdouble  y G_GNUC_UNUSED,
+			  gdouble  xscale G_GNUC_UNUSED,
+			  gdouble  yscale G_GNUC_UNUSED)
 {
-  cairo_surface_t *surface;
   GdkPixbuf *pixbuf = g_object_ref (fo_pixbuf_get_pixbuf (fo_image));
-  guchar *raw_image;
-  gboolean has_alpha;
-  gint rowstride, height, width;
-	
-  raw_image = gdk_pixbuf_get_pixels (pixbuf);
-  has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
-  rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-  height    = gdk_pixbuf_get_height (pixbuf);
-  width     = gdk_pixbuf_get_width (pixbuf);
-	
-  /*
-  cairo_translate (FO_DOC_CAIRO (fo_doc)->cr, 0, -height * yscale * 72 / PIXELS_PER_INCH);
-  cairo_scale (FO_DOC_CAIRO (fo_doc)->cr,
-	       width * xscale * 72 / PIXELS_PER_INCH,
-	       height * yscale * 72 / PIXELS_PER_INCH);
+  gint width = gdk_pixbuf_get_width (pixbuf);
+  gint height = gdk_pixbuf_get_height (pixbuf);
+  guchar *gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
+  int gdk_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+  int n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+  guchar *cairo_pixels;
+  cairo_format_t format;
+  cairo_surface_t *surface;
+  static const cairo_user_data_key_t key;
+  int j;
 
+  /*cairo_translate (FO_DOC_CAIRO (fo_doc)->cr, 0, height * yscale * 72.0 / PIXELS_PER_INCH);*/
+  cairo_save (FO_DOC_CAIRO(fo_doc)->cr);
+  /*
+  cairo_scale (FO_DOC_CAIRO (fo_doc)->cr,
+	       xscale * 72.0 / PIXELS_PER_INCH,
+	       yscale * 72.0 / PIXELS_PER_INCH);
   */
-  if (has_alpha)
+  if (n_channels == 3)
     {
-      surface = cairo_image_surface_create_for_data ((char *)raw_image,
-						     CAIRO_FORMAT_ARGB32,
-						     width,
-						     height,
-						     rowstride);
+      format = CAIRO_FORMAT_RGB24;
     }
   else
     {
-      surface = cairo_image_surface_create_for_data ((char *)raw_image,
-						     CAIRO_FORMAT_RGB24,
-						     width,
-						     height,
-						     rowstride);
+      format = CAIRO_FORMAT_ARGB32;
+    }
+
+  cairo_pixels = g_malloc (4 * width * height);
+  surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
+						 format,
+						 width, height, 4 * width);
+  cairo_surface_set_user_data (surface, &key,
+			       cairo_pixels, (cairo_destroy_func_t)g_free);
+
+  for (j = height; j; j--)
+    {
+      guchar *p = gdk_pixels;
+      guchar *q = cairo_pixels;
+
+      if (n_channels == 3)
+	{
+	  guchar *end = p + 3 * width;
+	  
+	  while (p < end)
+	    {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+	      q[0] = p[2];
+	      q[1] = p[1];
+	      q[2] = p[0];
+#else	  
+	      q[1] = p[0];
+	      q[2] = p[1];
+	      q[3] = p[2];
+#endif
+	      p += 3;
+	      q += 4;
+	    }
+	}
+      else
+	{
+	  guchar *end = p + 4 * width;
+	  guint t1,t2,t3;
+	    
+#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
+
+	  while (p < end)
+	    {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+	      MULT(q[0], p[2], p[3], t1);
+	      MULT(q[1], p[1], p[3], t2);
+	      MULT(q[2], p[0], p[3], t3);
+	      q[3] = p[3];
+#else	  
+	      q[0] = p[3];
+	      MULT(q[1], p[0], p[3], t1);
+	      MULT(q[2], p[1], p[3], t2);
+	      MULT(q[3], p[2], p[3], t3);
+#endif
+	      
+	      p += 4;
+	      q += 4;
+	    }
+	  
+#undef MULT
+	}
+
+      gdk_pixels += gdk_rowstride;
+      cairo_pixels += 4 * width;
     }
 
-  /*
   cairo_surface_set_fallback_resolution (surface,
 					 PIXELS_PER_INCH,
 					 PIXELS_PER_INCH);
-  */
   cairo_set_source_surface (FO_DOC_CAIRO (fo_doc)->cr, surface, 0, 0);
+  cairo_pattern_t *pattern = cairo_get_source (FO_DOC_CAIRO (fo_doc)->cr);
+  cairo_pattern_set_extend (pattern,
+			    CAIRO_EXTEND_NONE);
   cairo_paint (FO_DOC_CAIRO (fo_doc)->cr);
   cairo_surface_destroy (surface);
   g_object_unref (pixbuf);
+  cairo_restore (FO_DOC_CAIRO(fo_doc)->cr);
 }
 
 void
@@ -1084,7 +1136,7 @@ fo_doc_cairo_do_run_callbacks (cairo_t *cr,
 
       if (attr_type == libfo_pango_attr_callback_get_type ())
 	{
-	  GValue values[1] = { { 0, } };
+	  GValue values[1] = { { 0, { {0}, {0} } } };
 
 	  g_value_init (&values[0], G_TYPE_POINTER);
 	  g_value_set_pointer (&values[0], cr);
@@ -1185,9 +1237,9 @@ fo_doc_cairo_do_callbacks (cairo_t     *cr,
       baseline = pango_layout_iter_get_baseline (iter);
       
       fo_doc_cairo_do_line_callbacks (cr,
-				   line,
-				   x + logical_rect.x,
-				   y - baseline);
+				      line,
+				      x + logical_rect.x,
+				      y - baseline);
 
       if (line_number >= line_last)
 	{
@@ -1237,6 +1289,7 @@ fo_doc_cairo_render_layout_lines (FoDoc   *fo_doc,
     }
 
   gint line_last = fo_area_layout_get_line_last (area_layout);
+#if ENABLE_CLIP
   gdouble y2 =
       y1 - fo_area_layout_get_line_height (area_layout,
 					   line_last);
@@ -1244,28 +1297,27 @@ fo_doc_cairo_render_layout_lines (FoDoc   *fo_doc,
   gdouble x2 =
       x + fo_area_area_get_width (area_layout);
 
-#if ENABLE_CLIP
-  cairo_save (FO_DOC_CAIRO(fo_doc)->cr);
+  cairo_save (FO_DOC_CAIRO (fo_doc)->cr);
 
-  cairo_move_to (FO_DOC_CAIRO(fo_doc)->cr,
-		 x, FO_DOC_CAIRO(fo_doc)->page_height - y);
+  cairo_move_to (FO_DOC_CAIRO (fo_doc)->cr,
+		 x, FO_DOC_CAIRO (fo_doc)->page_height - y);
+  cairo_line_to (FO_DOC_CAIRO (fo_doc)->cr,
+		 x2, FO_DOC_CAIRO (fo_doc)->page_height - y);
+  cairo_line_to (FO_DOC_CAIRO (fo_doc)->cr,
+		 x2, FO_DOC_CAIRO (fo_doc)->page_height - y2);
   cairo_line_to (FO_DOC_CAIRO(fo_doc)->cr,
-		 x2, FO_DOC_CAIRO(fo_doc)->page_height - y);
-  cairo_line_to (FO_DOC_CAIRO(fo_doc)->cr,
-		 x2, FO_DOC_CAIRO(fo_doc)->page_height - y2);
-  cairo_line_to (FO_DOC_CAIRO(fo_doc)->cr,
-		 x, FO_DOC_CAIRO(fo_doc)->page_height - y2);
+		 x, FO_DOC_CAIRO (fo_doc)->page_height - y2);
 
-  cairo_clip (FO_DOC_CAIRO(fo_doc)->cr);
+  cairo_clip (FO_DOC_CAIRO (fo_doc)->cr);
 #endif
 
-  cairo_move_to (FO_DOC_CAIRO(fo_doc)->cr,
+  cairo_move_to (FO_DOC_CAIRO (fo_doc)->cr,
 		 x,
 		 FO_DOC_CAIRO (fo_doc)->page_height - y1);
-  pango_cairo_show_layout (FO_DOC_CAIRO(fo_doc)->cr,
+  pango_cairo_show_layout (FO_DOC_CAIRO (fo_doc)->cr,
 			   fo_layout_get_pango_layout (fo_area_layout_get_layout (area_layout)));
 
-  fo_doc_cairo_do_callbacks (FO_DOC_CAIRO(fo_doc)->cr,
+  fo_doc_cairo_do_callbacks (FO_DOC_CAIRO (fo_doc)->cr,
 			     fo_layout_get_pango_layout (fo_area_layout_get_layout (area_layout)),
 			     line_first,
 			     line_last,
@@ -1296,10 +1348,10 @@ fo_doc_cairo_render_layout (FoDoc   *fo_doc,
   g_return_if_fail (FO_DOC_CAIRO (fo_doc)->cr != NULL);
   g_return_if_fail (FO_IS_AREA_LAYOUT (area_layout));
 
-  cairo_move_to (FO_DOC_CAIRO(fo_doc)->cr,
+  cairo_move_to (FO_DOC_CAIRO (fo_doc)->cr,
 		 x,
 		 y);
-  pango_cairo_show_layout (FO_DOC_CAIRO(fo_doc)->cr,
+  pango_cairo_show_layout (FO_DOC_CAIRO (fo_doc)->cr,
 			   fo_layout_get_pango_layout (fo_area_layout_get_layout (area_layout)));
 
 }
@@ -1334,14 +1386,6 @@ fo_doc_cairo_debug_dump (FoObject *object,
 
   g_free (object_sprintf);
 
-  g_log (G_LOG_DOMAIN,
-	 G_LOG_LEVEL_DEBUG,
-	 "%s    job:     %s",
-	 indent,
-	 object_sprintf);
-
-  g_free (object_sprintf);
-
   object_sprintf = fo_object_debug_sprintf (fo_doc_cairo->cr);
 
   g_log (G_LOG_DOMAIN,
@@ -1352,14 +1396,6 @@ fo_doc_cairo_debug_dump (FoObject *object,
 
   g_free (object_sprintf);
 
-  g_log (G_LOG_DOMAIN,
-	 G_LOG_LEVEL_DEBUG,
-	 "%s    config:  %s",
-	 indent,
-	 object_sprintf);
-
-  g_free (object_sprintf);
-
   g_log (G_LOG_DOMAIN,
 	 G_LOG_LEVEL_DEBUG,
 	 "%s    base_filename: %s",
