@@ -92,7 +92,6 @@ static void          fo_doc_rect_filled_default     (FoDoc        *fo_doc,
 						     gdouble        height);
 static void          fo_doc_fill_default            (FoDoc        *fo_doc);
 static void          fo_doc_stroke_default          (FoDoc        *fo_doc);
-static void          fo_doc_fill_stroke_default     (FoDoc        *fo_doc);
 static void          fo_doc_place_image_default     (FoDoc        *fo_doc,
 						     FoImage      *fo_image,
 						     gdouble        x,
@@ -202,7 +201,6 @@ fo_doc_base_init (FoDocClass *klass)
   klass->rect_filled         = fo_doc_rect_filled_default;
   klass->fill                = fo_doc_fill_default;
   klass->stroke              = fo_doc_stroke_default;
-  klass->fill_stroke         = fo_doc_fill_stroke_default;
   klass->place_image         = fo_doc_place_image_default;
   klass->render_layout_lines = fo_doc_render_layout_lines_default;
   klass->render_layout       = fo_doc_render_layout_default;
@@ -272,9 +270,9 @@ fo_doc_new (void)
 
 void
 fo_doc_open_file_default (FoDoc          *fo_doc,
-			  const gchar    *filename,
-			  FoLibfoContext *libfo_context,
-			  GError        **error)
+			  const gchar    *filename G_GNUC_UNUSED,
+			  FoLibfoContext *libfo_context G_GNUC_UNUSED,
+			  GError        **error G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -291,7 +289,8 @@ fo_doc_open_file_default (FoDoc          *fo_doc,
  * @libfo_context: #FoLibfoContext containing extra info.
  * @error:         Information about any error that occurred.
  * 
- * 
+ *
+ * No reference to @libfo_context is kept.
  **/
 void
 fo_doc_open_file (FoDoc          *fo_doc,
@@ -536,9 +535,9 @@ fo_doc_set_base_dir (FoDoc      *fo_doc,
  **/
 void
 fo_doc_begin_page_default (FoDoc   *fo_doc,
-			   gdouble  width,
-			   gdouble  height,
-			   GError **error)
+			   gdouble  width G_GNUC_UNUSED,
+			   gdouble  height G_GNUC_UNUSED,
+			   GError **error G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -644,7 +643,7 @@ fo_doc_get_fill_color (FoDoc        *fo_doc)
  **/
 void
 fo_doc_set_fill_color_default (FoDoc        *fo_doc,
-			       FoDatatype   *color)
+			       FoDatatype   *color G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -714,7 +713,7 @@ fo_doc_get_stroke_color (FoDoc        *fo_doc)
  **/
 void
 fo_doc_set_stroke_color_default (FoDoc        *fo_doc,
-				 FoDatatype   *color)
+				 FoDatatype   *color G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -783,7 +782,7 @@ fo_doc_get_line_cap (FoDoc        *fo_doc)
  **/
 void
 fo_doc_set_line_cap_default (FoDoc        *fo_doc,
-			     FoDocLineCap  line_cap)
+			     FoDocLineCap  line_cap G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -852,7 +851,7 @@ fo_doc_get_line_join (FoDoc        *fo_doc)
  **/
 void
 fo_doc_set_line_join_default (FoDoc        *fo_doc,
-			      FoDocLineJoin line_join)
+			      FoDocLineJoin line_join G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -921,7 +920,7 @@ fo_doc_get_line_width (FoDoc        *fo_doc)
  **/
 void
 fo_doc_set_line_width_default (FoDoc        *fo_doc,
-			       gdouble        line_width)
+			       gdouble        line_width G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -963,8 +962,8 @@ fo_doc_set_line_width (FoDoc        *fo_doc,
  **/
 void
 fo_doc_set_dash_default (FoDoc        *fo_doc,
-			 gdouble        b,
-			 gdouble        w)
+			 gdouble        b G_GNUC_UNUSED,
+			 gdouble        w G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1011,8 +1010,8 @@ fo_doc_set_dash (FoDoc        *fo_doc,
  **/
 void
 fo_doc_translate_default (FoDoc        *fo_doc,
-			  gdouble        x,
-			  gdouble        y)
+			  gdouble        x G_GNUC_UNUSED,
+			  gdouble        y G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1139,8 +1138,8 @@ fo_doc_restore (FoDoc        *fo_doc)
  **/
 void
 fo_doc_line_to_default (FoDoc        *fo_doc,
-			gdouble        x,
-			gdouble        y)
+			gdouble        x G_GNUC_UNUSED,
+			gdouble        y G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1178,8 +1177,8 @@ fo_doc_line_to (FoDoc        *fo_doc,
  **/
 void
 fo_doc_move_to_default (FoDoc        *fo_doc,
-			gdouble        x,
-			gdouble        y)
+			gdouble        x G_GNUC_UNUSED,
+			gdouble        y G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1219,10 +1218,10 @@ fo_doc_move_to (FoDoc        *fo_doc,
  **/
 void
 fo_doc_line_stroked_default (FoDoc *fo_doc,
-			     gdouble x0,
-			     gdouble y0,
-			     gdouble x1,
-			     gdouble y1)
+			     gdouble x0 G_GNUC_UNUSED,
+			     gdouble y0 G_GNUC_UNUSED,
+			     gdouble x1 G_GNUC_UNUSED,
+			     gdouble y1 G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1269,10 +1268,10 @@ fo_doc_line_stroked (FoDoc *fo_doc,
  **/
 void
 fo_doc_rect_stroked_default (FoDoc        *fo_doc,
-			     gdouble        x,
-			     gdouble        y,
-			     gdouble        width,
-			     gdouble        height)
+			     gdouble        x G_GNUC_UNUSED,
+			     gdouble        y G_GNUC_UNUSED,
+			     gdouble        width G_GNUC_UNUSED,
+			     gdouble        height G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1318,10 +1317,10 @@ fo_doc_rect_stroked (FoDoc        *fo_doc,
  **/
 void
 fo_doc_rect_filled_default (FoDoc        *fo_doc,
-			     gdouble        x,
-			     gdouble        y,
-			     gdouble        width,
-			     gdouble        height)
+			     gdouble        x G_GNUC_UNUSED,
+			     gdouble        y G_GNUC_UNUSED,
+			     gdouble        width G_GNUC_UNUSED,
+			     gdouble        height G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1413,37 +1412,6 @@ fo_doc_stroke (FoDoc        *fo_doc)
   FO_DOC_GET_CLASS (fo_doc)->stroke (fo_doc);
 }
 
-/**
- * fo_doc_fill_stroke_default:
- * @fo_doc: #FoDoc.
- * 
- * Fill and stroke the path of @fo_doc with the current fill and
- * stroke colors.
- **/
-void
-fo_doc_fill_stroke_default (FoDoc        *fo_doc)
-{
-#if defined(LIBFO_DEBUG)
-  g_log (G_LOG_DOMAIN,
-	 G_LOG_LEVEL_DEBUG,
-	 _("%s does not have a 'fill_stroke' function."),
-	 fo_object_sprintf (fo_doc));
-#endif
-}
-
-/**
- * fo_doc_fill_stroke:
- * @fo_doc: #FoDoc.
- * 
- * Fill and stroke the path of @fo_doc with the current fill and
- * stroke colors.
- **/
-void
-fo_doc_fill_stroke (FoDoc *fo_doc)
-{
-  FO_DOC_GET_CLASS (fo_doc)->fill_stroke (fo_doc);
-}
-
 /**
  * fo_doc_place_image_default:
  * @fo_doc:   #FoDoc in which to place image
@@ -1458,11 +1426,11 @@ fo_doc_fill_stroke (FoDoc *fo_doc)
  **/
 void
 fo_doc_place_image_default (FoDoc   *fo_doc,
-			    FoImage *image,
-			    gdouble   x,
-			    gdouble   y,
-			    gdouble   xscale,
-			    gdouble   yscale)
+			    FoImage *image G_GNUC_UNUSED,
+			    gdouble   x G_GNUC_UNUSED,
+			    gdouble   y G_GNUC_UNUSED,
+			    gdouble   xscale G_GNUC_UNUSED,
+			    gdouble   yscale G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1512,9 +1480,9 @@ fo_doc_place_image (FoDoc   *fo_doc,
  **/
 void
 fo_doc_render_layout_lines_default (FoDoc   *fo_doc,
-				    FoArea  *area_layout,
-				    gdouble   x,
-				    gdouble   y)
+				    FoArea  *area_layout G_GNUC_UNUSED,
+				    gdouble   x G_GNUC_UNUSED,
+				    gdouble   y G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
@@ -1558,9 +1526,9 @@ fo_doc_render_layout_lines (FoDoc   *fo_doc,
  **/
 void
 fo_doc_render_layout_default (FoDoc   *fo_doc,
-			      FoArea  *area_layout,
-			      gdouble   x,
-			      gdouble   y)
+			      FoArea  *area_layout G_GNUC_UNUSED,
+			      gdouble   x G_GNUC_UNUSED,
+			      gdouble   y G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG)
   g_log (G_LOG_DOMAIN,
