@@ -202,16 +202,24 @@ fo_xsl_formatter_finalize  (GObject *object)
   fo_xsl_formatter = FO_XSL_FORMATTER (object);
 
   if (fo_xsl_formatter->result_tree != NULL)
-    fo_xml_doc_unref (fo_xsl_formatter->result_tree);
+    {
+      fo_xml_doc_unref (fo_xsl_formatter->result_tree);
+    }
 
   if (fo_xsl_formatter->fo_tree != NULL)
-    g_object_unref (fo_xsl_formatter->fo_tree);
+    {
+      g_object_unref (fo_xsl_formatter->fo_tree);
+    }
 
   if (fo_xsl_formatter->area_tree != NULL)
-    g_object_unref (fo_xsl_formatter->area_tree);
+    {
+      g_object_unref (fo_xsl_formatter->area_tree);
+    }
 
   if (fo_xsl_formatter->fo_doc != NULL)
-    g_object_unref (fo_xsl_formatter->fo_doc);
+    {
+      g_object_unref (fo_xsl_formatter->fo_doc);
+    }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -320,12 +328,12 @@ fo_xml_element_node_to_fo_node (xmlNodePtr node,
 {
   FoFo *fo_node = NULL;
   gboolean ok = FALSE;
-  xmlChar *name;
+  gchar *name;
 
   g_return_val_if_fail (node != NULL, NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  name = (xmlChar*) node->name;
+  name = (gchar *) node->name;
 
   /* Hand-built parsing of node name to determine FoFo type. */
   switch (name[0]) {
@@ -650,7 +658,8 @@ fo_new_xml_to_fo_tree (xmlNodePtr node,
     {
     case XML_ELEMENT_NODE:
       if ((node->ns != NULL) &&
-	  (strcmp (node->ns->href, XSL_FO_NAMESPACE) == 0 ))
+	  (strcmp ((gchar *) node->ns->href,
+		   XSL_FO_NAMESPACE) == 0 ))
 	{
 	  fo_node = fo_xml_element_node_to_fo_node (node,
 						    &tmp_error);
@@ -700,20 +709,18 @@ fo_new_xml_to_fo_tree (xmlNodePtr node,
       break;
     case XML_TEXT_NODE:
       {
-	xmlChar *text = xmlNodeGetContent (node);
+	gchar *text = (gchar *) xmlNodeGetContent (node);
 
 	if (fo_fo_get_allow_mixed_content (parent_fo))
 	  {
 	    fo_text = fo_text_new ();
-	    fo_text_set_value (fo_text, text);
-	    fo_fo_update_from_context (fo_text,
-				       fo_fo_get_context (parent_fo));
 	    fo_node_append (FO_NODE (parent_fo),
 			    FO_NODE (fo_text));
+	    fo_text_set_value (fo_text, text);
 	  }
 	else
 	  {
-	    xmlChar *text_ptr = text;
+	    gchar *text_ptr = text;
 	    gboolean whitespace_only = TRUE;
 
 	    while (*text_ptr)
