@@ -277,7 +277,7 @@ fo_fo_finalize (GObject *object)
 void
 fo_fo_set_property (GObject      *object,
 		    guint         param_id,
-		    const GValue *value,
+		    const GValue *value G_GNUC_UNUSED,
 		    GParamSpec   *pspec)
 {
   G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -658,8 +658,8 @@ fo_fo_sprintf (FoObject *object)
  * 'update_from_context' function.
  **/
 void
-fo_fo_update_from_context_default (FoFo *fo,
-				   FoContext *context)
+fo_fo_update_from_context_default (FoFo      *fo,
+				   FoContext *context G_GNUC_UNUSED)
 {
   g_log (G_LOG_DOMAIN,
 	 G_LOG_LEVEL_DEBUG,
@@ -703,8 +703,8 @@ fo_fo_update_from_context (FoFo      *fo_fo,
  * Return value: FALSE
  **/
 gboolean
-fo_fo_validate_content_default (FoFo *fo,
-				GError **error)
+fo_fo_validate_content_default (FoFo    *fo G_GNUC_UNUSED,
+				GError **error G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG) && 0
   g_log (G_LOG_DOMAIN,
@@ -765,6 +765,17 @@ fo_fo_validate (FoFo *fo,
 				     error);
 }
 
+/**
+ * fo_fo_validate_default:
+ * @fo:              #FoFo to validate.
+ * @current_context: #FoContext of @fo.
+ * @parent_context:  #FoContext of parent of @fo.
+ * 
+ * Default validation is to just merge and update contexts.
+ *
+ * This will only be called for #FoFo that do not have either a
+ * _validate2 or _validate function of their own.
+ **/
 void
 fo_fo_validate_default (FoFo *fo,
 			FoContext *current_context,
@@ -784,11 +795,21 @@ fo_fo_validate_default (FoFo *fo,
   */
 }
 
+/**
+ * fo_fo_validate2_default:
+ * @fo:              #FoFo to validate.
+ * @current_context: #FoContext of @fo.
+ * @parent_context:  #FoContext of parent of @fo.
+ * @error:           #GError indicating any error that occurs
+ * 
+ * Default behaviour is to fallback to using the FO's _validate
+ * function.
+ **/
 void
 fo_fo_validate2_default (FoFo *fo,
 			 FoContext *current_context,
 			 FoContext *parent_context,
-			 GError   **error)
+			 GError   **error G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG) && 0
   g_log (G_LOG_DOMAIN,
@@ -1008,14 +1029,9 @@ gboolean
 fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 					   gpointer    data)
 {
-  FoPropertyResolveContext *prop_context = (FoPropertyResolveContext *) data;
-  xmlNodePtr element;
-  xslAttrListIteratorPtr iterator;
-  FoProperty *font_size = NULL;
-  FoFo *fo_fo;
+  FoPropertyResolveContext *prop_context =
+    (FoPropertyResolveContext *) data;
   GSList *node_properties = NULL;
-  FoContext *current_context;
-  FoContext *parent_context;
   GError *error = NULL;
   FoArea *reference_area = prop_context->reference_area;
   FoWarningFlag warning_mode = prop_context->warning_mode;
@@ -1031,10 +1047,10 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 	     fo_object_debug_sprintf (reference_area));
 #endif
 
-  fo_fo = FO_FO (fo_node);
+  FoFo *fo_fo = FO_FO (fo_node);
 
-  element = fo_xml_node_get_xml_node (fo_fo->element);
-  parent_context =
+  xmlNodePtr element = fo_xml_node_get_xml_node (fo_fo->element);
+  FoContext *parent_context =
     fo_fo_get_context (FO_FO (fo_node_parent (FO_NODE (fo_fo))));
 
   /* Special processing for FOs with special behaviour defined by the
@@ -1045,7 +1061,8 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
       xmlAttrPtr number_columns_spanned_attr = NULL;
       xmlAttrPtr number_rows_spanned_attr = NULL;
 
-      column_number_attr = xmlHasProp (element, "column-number");
+      column_number_attr = xmlHasProp (element,
+				       (xmlChar *) "column-number");
       if (column_number_attr)
 	{
 	  FoTypeFunc type_func =
@@ -1055,17 +1072,19 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 	  FoPropertyClass *property_class =
 	    g_type_class_ref (type_func());
 
-	  const xmlChar *property_expression =
+	  gchar *property_expression = (gchar *)
 	    xmlNodeGetContent ((xmlNodePtr) column_number_attr);
 
 	  FoProperty *column_number =
 	    fo_property_new_from_expr (property_class,
-				       property_expression,
+				       (const gchar *) property_expression,
 				       parent_context,
 				       NULL,
 				       fo_fo,
 				       &error);
 
+	  xmlFree (property_expression);
+
 	  if (error != NULL)
 	    {
 	      fo_object_maybe_propagate_error (FO_OBJECT (fo_fo),
@@ -1082,7 +1101,8 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 	}
 
       number_columns_spanned_attr =
-	xmlHasProp (element, "number-columns-spanned");
+	xmlHasProp (element,
+		    (xmlChar *) "number-columns-spanned");
       if (number_columns_spanned_attr)
 	{
 	  FoTypeFunc type_func =
@@ -1092,17 +1112,19 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 	  FoPropertyClass *property_class =
 	    g_type_class_ref (type_func());
 
-	  const xmlChar *property_expression =
+	  gchar *property_expression = (gchar *)
 	    xmlNodeGetContent ((xmlNodePtr) number_columns_spanned_attr);
 
 	  FoProperty *number_columns_spanned =
 	    fo_property_new_from_expr (property_class,
-				       property_expression,
+				       (const gchar *) property_expression,
 				       parent_context,
 				       NULL,
 				       fo_fo,
 				       &error);
 
+	  xmlFree (property_expression);
+
 	  if (error != NULL)
 	    {
 	      fo_object_maybe_propagate_error (FO_OBJECT (fo_fo),
@@ -1118,7 +1140,10 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 						    number_columns_spanned);
 	}
 
-      number_rows_spanned_attr = xmlHasProp (element, "number-rows-spanned");
+      number_rows_spanned_attr =
+	xmlHasProp (element,
+		    (xmlChar *) "number-rows-spanned");
+
       if (number_rows_spanned_attr)
 	{
 	  FoTypeFunc type_func =
@@ -1128,17 +1153,19 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 	  FoPropertyClass *property_class =
 	    g_type_class_ref (type_func());
 
-	  const xmlChar *property_expression =
+	  gchar *property_expression = (gchar *)
 	    xmlNodeGetContent ((xmlNodePtr) number_rows_spanned_attr);
 
 	  FoProperty *number_rows_spanned =
 	    fo_property_new_from_expr (property_class,
-				       property_expression,
+				       (const gchar *) property_expression,
 				       parent_context,
 				       NULL,
 				       fo_fo,
 				       &error);
 
+	  xmlFree (property_expression);
+
 	  if (error != NULL)
 	    {
 	      fo_object_maybe_propagate_error (FO_OBJECT (fo_fo),
@@ -1169,11 +1196,14 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
       fo_table_cell_resolve_column (fo_fo);
     }
 
-  /* Handle the 'font-size' property first since it's value is part of
+  /* Handle the 'font-size' property first since its value is part of
      the context for evaluating all other properties. */
-  if (xmlHasProp (element, "font-size"))
+  FoProperty *font_size = NULL;
+  if (xmlHasProp (element, (xmlChar *) "font-size"))
     {
-      xmlAttrPtr font_size_attr = xmlHasProp (element, "font-size");
+      xmlAttrPtr font_size_attr =
+	xmlHasProp (element,
+		    (xmlChar *) "font-size");
 
       FoTypeFunc type_func =
 	g_hash_table_lookup (prop_eval_hash,
@@ -1182,14 +1212,19 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
       FoPropertyClass *property_class =
 	g_type_class_ref (type_func());
 
+      gchar *property_expression = (gchar *)
+	xmlNodeGetContent ((xmlNodePtr) font_size_attr);
+
       font_size =
 	fo_property_new_from_expr (property_class,
-				   xmlNodeGetContent ((xmlNodePtr) font_size_attr),
+				   (const gchar *) property_expression,
 				   parent_context,
 				   NULL,
 				   fo_fo,
 				   &error);
 
+      xmlFree (property_expression);
+
       if (error != NULL)
 	{
 	  fo_object_maybe_propagate_error (FO_OBJECT (fo_fo),
@@ -1206,7 +1241,7 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
     }
 
 
-  iterator = xslAttrListGetIterator (element);
+  xslAttrListIteratorPtr iterator = xslAttrListGetIterator (element);
 
   /* Iterate over the remaining attributes. */
   while (iterator)
@@ -1214,13 +1249,14 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
       FoProperty *property = NULL;
 
       /* Skip processing attributes that have already been handled. */
-      if (strcmp (xslAttrListIteratorName (iterator), "font-size") == 0 ||
+      if (strcmp ((gchar *) xslAttrListIteratorName (iterator),
+		  "font-size") == 0 ||
 	  (FO_IS_TABLE_CELL (fo_fo) &&
-	   (strcmp (xslAttrListIteratorName (iterator),
+	   (strcmp ((gchar *) xslAttrListIteratorName (iterator),
 		    "column-number") == 0 ||
-	    strcmp (xslAttrListIteratorName (iterator),
+	    strcmp ((gchar *) xslAttrListIteratorName (iterator),
 		    "number-columns-spanned") == 0 ||
-	    strcmp (xslAttrListIteratorName (iterator),
+	    strcmp ((gchar *) xslAttrListIteratorName (iterator),
 		    "number-rows-spanned") == 0)))
 	{
 	  if (xslAttrListIteratorNext (iterator))
@@ -1243,9 +1279,12 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 	  property_class =
 	    g_type_class_ref (type_func());
 
+	  const gchar *property_expression =
+	    (const gchar *) xslAttrListIteratorValue (iterator);
+
 	  property =
 	    fo_property_new_from_expr (property_class,
-				       xslAttrListIteratorValue (iterator),
+				       property_expression,
 				       parent_context,
 				       font_size ? font_size : NULL,
 				       fo_fo,
@@ -1284,7 +1323,7 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 
   xslAttrListIteratorDestroy (iterator);
 
-  current_context = fo_context_new ();
+  FoContext *current_context = fo_context_new ();
   if (node_properties != NULL)
     {
       fo_context_update_from_slist (current_context, node_properties);
@@ -1293,7 +1332,7 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 
   fo_context_util_spaces_resolve (current_context, reference_area);
   fo_context_util_length_ranges_resolve (current_context, reference_area);
-  /* Validate property values and property interrations of FO tree. */
+  /* Validate property values and property interactions of FO tree. */
   fo_fo_validate (fo_fo,
 		  current_context,
 		  parent_context,
@@ -1301,9 +1340,8 @@ fo_fo_resolve_property_attributes_default (FoNode     *fo_node,
 
   if (error != NULL)
     {
-      g_propagate_error (/*FO_OBJECT (fo_fo),*/
-					prop_context->error,
-					error);
+      g_propagate_error (prop_context->error,
+			 error);
       return TRUE;
     }
 
@@ -1368,13 +1406,7 @@ fo_fo_children_properties_resolve_default (FoFo         *this_fo,
 					   FoWarningFlag warning_mode,
 					   GError      **error)
 {
-  FoNode *child_node;
   GError *tmp_error = NULL;
-  FoArea *this_fo_area = NULL;
-  FoArea *child_fo_parent_area;
-  FoFoAreaNew2Context area_new2_context;
-  FoPropertyResolveContext prop_context;
-  gboolean resolve_attributes_halted;
 
   g_return_if_fail (FO_IS_FO (this_fo));
   g_return_if_fail (FO_IS_AREA (this_fo_parent_area));
@@ -1387,6 +1419,7 @@ fo_fo_children_properties_resolve_default (FoFo         *this_fo,
 	     fo_object_debug_sprintf (this_fo_parent_area));
 #endif
 
+  FoPropertyResolveContext prop_context;
   prop_context.reference_area       = fo_area_get_reference (this_fo_parent_area);
   prop_context.prop_eval_hash       = prop_eval_hash;
   prop_context.continue_after_error = continue_after_error;
@@ -1394,7 +1427,7 @@ fo_fo_children_properties_resolve_default (FoFo         *this_fo,
   prop_context.warning_mode         = warning_mode;
   prop_context.error                = &tmp_error;
 
-  resolve_attributes_halted =
+  gboolean resolve_attributes_halted =
     fo_fo_resolve_property_attributes (FO_NODE (this_fo),
 				       &prop_context);
 
@@ -1406,6 +1439,8 @@ fo_fo_children_properties_resolve_default (FoFo         *this_fo,
       return;
     }
 
+  FoArea *this_fo_area = NULL;
+  FoFoAreaNew2Context area_new2_context;
   area_new2_context.fo_doc               = fo_doc;
   area_new2_context.parent_area          = this_fo_parent_area;
   area_new2_context.new_area             = &this_fo_area;
@@ -1418,6 +1453,7 @@ fo_fo_children_properties_resolve_default (FoFo         *this_fo,
 
   *new_area = this_fo_area;
 
+  FoArea *child_fo_parent_area;
   if (*new_area != NULL)
     {
       child_fo_parent_area = *new_area;
@@ -1432,8 +1468,7 @@ fo_fo_children_properties_resolve_default (FoFo         *this_fo,
 	     fo_object_debug_sprintf (*new_area));
 #endif
 
-  child_node = fo_node_first_child (FO_NODE (this_fo));
-
+  FoNode *child_node = fo_node_first_child (FO_NODE (this_fo));
   while (child_node)
     {
       FoArea *area = NULL;
@@ -1557,10 +1592,10 @@ fo_fo_area_new (FoFo    *fo,
  **/
 static void
 fo_fo_area_new_default (FoFo    *fo,
-			FoDoc   *fo_doc,
-			FoArea  *parent_area,
+			FoDoc   *fo_doc G_GNUC_UNUSED,
+			FoArea  *parent_area G_GNUC_UNUSED,
 			FoArea **new_area,
-			guint    debug_level)
+			guint    debug_level G_GNUC_UNUSED)
 {
   *new_area = NULL;
 
@@ -1594,9 +1629,9 @@ void fo_fo_area_new2 (FoFo *fo,
 				     error);
 }
 
-void fo_fo_area_new2_default (FoFo *fo,
-			      FoFoAreaNew2Context *context,
-			      GError **error)
+void fo_fo_area_new2_default (FoFo *fo G_GNUC_UNUSED,
+			      FoFoAreaNew2Context *context G_GNUC_UNUSED,
+			      GError **error G_GNUC_UNUSED)
 {
 #if defined(LIBFO_DEBUG) && 0
   g_log (G_LOG_DOMAIN,
@@ -1812,16 +1847,17 @@ fo_fo_validate_block_or_whitespace (FoNode *fo_node,
 
 /**
  * fo_fo_validate_pcdata_inline_block_neutral:
- * @fo_node:                            #FoFo to validate
- * @is_not_pcdata_inline_block_neutral: Pointer to #gboolean storing result
+ * @fo_node: #FoFo to validate
+ * @data:    Pointer to #gboolean storing result
  * 
  * Validates that the content of @fo_node is only ##PCDATA or #FoInline or
  * #FoNeutral formatting objects.
  **/
 void
-fo_fo_validate_pcdata_inline_block_neutral (FoNode *fo_node,
-					    gboolean *is_not_pcdata_inline_block_neutral)
+fo_fo_validate_pcdata_inline_block_neutral (FoNode  *fo_node,
+					    gpointer data)
 {
+  gboolean *is_not_pcdata_inline_block_neutral = (gboolean *) data;
   if (!FO_IS_INLINE_FO (FO_FO (fo_node)) &&
       !FO_IS_NEUTRAL_FO (FO_FO (fo_node)) &&
       !FO_IS_BLOCK_FO (FO_FO (fo_node)))
@@ -1921,7 +1957,10 @@ fo_fo_register_id (FoFo       *fo,
   g_return_if_fail (FO_IS_FO (fo));
   g_return_if_fail (FO_IS_ID (id));
 
-  fo_tree_id_add (fo->tree,
-		  fo_id_get_value (id),
-		  fo);
+  if (fo->tree != NULL)
+    {
+      fo_tree_id_add (fo->tree,
+		      fo_id_get_value (id),
+		      fo);
+    }
 }
