@@ -188,6 +188,12 @@ fo_property_finalize (GObject *object)
 
   property = FO_PROPERTY (object);
 
+  if (property->value != NULL)
+    {
+      g_object_unref (property->value);
+      property->value = NULL;
+    }
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -356,8 +362,8 @@ fo_property_set_value_default (FoProperty *property,
  * Return value: 
  **/
 FoDatatype*
-fo_property_resolve_enum_default (const gchar *token,
-				  FoContext   *context,
+fo_property_resolve_enum_default (const gchar *token G_GNUC_UNUSED,
+				  FoContext   *context G_GNUC_UNUSED,
 				  GError     **error)
 {
   g_warning ("This property doesn't have any enumerated values.\n");
@@ -385,10 +391,10 @@ fo_property_resolve_enum_default (const gchar *token,
  **/
 FoDatatype*
 fo_property_resolve_percent_default (gdouble            percentage,
-				     const FoDatatype  *font_size,
-				     const FoFo        *fo_node,
-				     const FoContext   *context,
-				     GError           **err)
+				     const FoDatatype  *font_size G_GNUC_UNUSED,
+				     const FoFo        *fo_node G_GNUC_UNUSED,
+				     const FoContext   *context G_GNUC_UNUSED,
+				     GError           **err G_GNUC_UNUSED)
 {
   FoDatatype *percentage_datatype;
 
@@ -421,7 +427,7 @@ fo_property_new_from_string (FoPropertyClass *property_class,
 			     const gchar *expr,
 			     FoContext   *context,
 			     FoProperty    *current_font_size,
-			     FoFo *fo_node,
+			     FoFo *fo_node G_GNUC_UNUSED,
 			     GError **error)
 {
   FoProperty *property;
@@ -534,13 +540,28 @@ fo_property_new_from_expr_default (FoPropertyClass *property_class,
   return property;
 }
 
+/**
+ * fo_property_new_from_expr:
+ * @property_class:    Class of property for which to evaluate expression.
+ * @expr:              Expression to evaluate.
+ * @context:           Current #FoContext.
+ * @current_font_size: Current "font-size" value
+ * @fo_node:           Current FO
+ * @error:             Indication of any error that occurred
+ * 
+ * Evaluates @expr as a value of an instance of @property_class.
+ * The other parameters provide the context for evaluating @expr.
+ * 
+ * Returns: New instance of @property_class, or %NULL if an
+ *          error ocurred.
+ **/
 FoProperty*
 fo_property_new_from_expr (FoPropertyClass *property_class,
-			   const gchar *expr,
-			   FoContext   *context,
-			   FoProperty    *current_font_size,
-			   FoFo *fo_node,
-			   GError **error)
+			   const gchar     *expr,
+			   FoContext       *context,
+			   FoProperty      *current_font_size,
+			   FoFo            *fo_node,
+			   GError         **error)
 {
   g_return_val_if_fail (FO_IS_PROPERTY_CLASS (property_class), NULL);
 
@@ -583,7 +604,7 @@ fo_property_class_is_inherited (FoPropertyClass *property_class)
 
 /**
  * fo_property_is_inherited:
- * @property: 
+ * @property: #FoProperty or subclass of #FoProperty
  * 
  * Indicates whether the XSL property represented by @property is
  * defined in the XSL 1.0 Recommendation as an inherited property.
@@ -617,7 +638,7 @@ fo_property_class_is_shorthand (FoPropertyClass *property_class)
 
 /**
  * fo_property_is_shorthand:
- * @property: 
+ * @property: #FoProperty or a subclass of #FoProperty.
  * 
  * Indicates whether the XSL property represented by @property is
  * defined in the XSL 1.0 Recommendation as a shorthand property.
